/*
 * ObjectAccessor.java
 * Created on 28. Mai 2009, 22:40
 *
 * This file is part of JAMS
 * Copyright (C) FSU Jena
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */
package jams.dataaccess;

import jams.data.Attribute;
import jams.data.JAMSData;
import jams.data.DefaultDataFactory;
import jams.data.JAMSEntity;
import jams.JAMS;
import jams.data.DataFactory;

/**
 *
 * @author Sven Kralisch <sven.kralisch at uni-jena.de>
 */
public class ObjectArrayAccessor implements DataAccessor {

    Attribute.ObjectArray componentObject;

    Attribute.ObjectArray[] entityObject;

    int index;

    int accessType;

    int updateType;

    public ObjectArrayAccessor(DataFactory dataFactory,  Attribute.Entity[] entities, JAMSData dataObject, String attributeName, int accessType, int updateType) throws JAMSEntity.NoSuchAttributeException {

        //get the entities' data objects
        entityObject = new Attribute.ObjectArray[entities.length];
        for (int i = 0; i < entities.length; i++) {
            if (entities[i].existsAttribute(attributeName)) {
                try {
                    entityObject[i] = (Attribute.ObjectArray) entities[i].getObject(attributeName);
                } catch (JAMSEntity.NoSuchAttributeException nsae) {
                }
            } else {
                if (accessType != DataAccessor.READ_ACCESS) {
                    entityObject[i] = dataFactory.createObjectArray();
                    entities[i].setObject(attributeName, entityObject[i]);
                } else {
                    throw new JAMSEntity.NoSuchAttributeException(JAMS.i18n("Attribute_") + attributeName + JAMS.i18n("_does_not_exist!"));
                }
            }
        }

        this.accessType = accessType;
        this.updateType = updateType;

        this.componentObject = (Attribute.ObjectArray) dataObject;
    }

    @Override
    public void initEntityData() {
        for (Attribute.ObjectArray array : entityObject) {
            if (componentObject.getValue() != null) {
                array.setValue((Object[]) componentObject.getValue().clone());
            }
        }
    }

    @Override
    public void setIndex(int index) {
        this.index = index;
    }

    @Override
    public void read() {
        componentObject.setValue(entityObject[index].getValue());
    }

    @Override
    public void write() {
        entityObject[index].setValue(componentObject.getValue());
    }

    @Override
    public int getAccessType() {
        return accessType;
    }

    @Override
    public int getUpdateType() {
        return updateType;
    }

    @Override
    public JAMSData getComponentObject() {
        return this.componentObject;
    }
}
